package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.feign.TmsTruckFeign;
import com.steerinfo.dil.mapper.QmsQueueResultMapper;
import com.steerinfo.dil.model.QmsQueueResult;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.dil.model.QmsQueueList;
import com.steerinfo.dil.mapper.QmsQueueListMapper;
import com.steerinfo.dil.service.IQmsQueueListService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import javax.xml.transform.Source;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * QmsQueueList服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-14 11:29
 * 类描述
 * 修订历史：
 * 日期：2021-09-14
 * 作者：generator
 * 参考：
 * 描述：QmsQueueList服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "qmsQueueListService")
public class QmsQueueListServiceImpl implements IQmsQueueListService {

    @Autowired
    private QmsQueueListMapper qmsQueueListMapper;

    @Autowired
    private QmsQueueResultMapper qmsQueueResultMapper;

    @Autowired
    private TmsTruckFeign tmsTruckFeign;

    /**
     * 开始排队  添加 链表排序
     * @param orderNumber 运输订单号
     * @return
     */
    @Transactional
    public int addQueueList(String orderNumber){
        QmsQueueList qmsQueueList = new QmsQueueList();
        Map<String, Object> map = new HashMap<>();
        map.put("orderNumber", orderNumber);
        //通过运输订单号查询 实绩中  网格Id 及 运力Id
        Map<String, Object> listResultMap = qmsQueueListMapper.getQueueResultByOrderNumber(map);
        map.putAll(listResultMap);
        //添加主键ID
        BigDecimal maxId = qmsQueueListMapper.selectMaxId();
        qmsQueueList.setListId(maxId);
        //添加网格Id
        BigDecimal gridId = DataChange.dataToBigDecimal(map.get("gridId"));
        qmsQueueList.setGridId(gridId);
        //添加运力Id
        qmsQueueList.setCapacityId(DataChange.dataToBigDecimal(map.get("capacityId")));
        //查询当前链表还在排队中的 最大序号 + 1
        Integer maxListNodeOrder = qmsQueueListMapper.getMaxListNodeOrder(map);
        int nodeOrder = (maxListNodeOrder == null ? 0 : maxListNodeOrder) + 1;
        //添加排队序号与逻辑删除 0： 正在排队
        qmsQueueList.setListNodeOrder(new BigDecimal(nodeOrder));
        qmsQueueList.setDeleted(new BigDecimal(0));

        qmsQueueList.setInsertUsername("admin");
        qmsQueueList.setInsertTime(new Date());
        int i = qmsQueueListMapper.insertSelective(qmsQueueList);
        //更新排队实绩表
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setListId(maxId);
        qmsQueueResult.setResultId(DataChange.dataToBigDecimal(map.get("resultId")));
        qmsQueueResult.setResultAcceptTime(new Date()); //添加接收时间
        qmsQueueResult.setResultStartTime(new Date()); //添加排队时间
        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        return i1 + i ;
    }

    /**
     * 排队转移
     * 参数：实绩Id、门岗名称||门岗ID 或者 月台名称||月台ID、转移原因
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int changeQueue(Map<String, Object> map) {
        //通过传入的实绩Id查询 链表Id 、运力Id
        Map<String, Object> mapValue = qmsQueueListMapper.getQueueResultByOrderNumber(map);
        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("resultId"));
        //获取链表ID
        BigDecimal listId = DataChange.dataToBigDecimal(mapValue.get("listId"));
        //修改原链表状态 1
        updateQueueListToDie(listId);
        //获取运力ID
        BigDecimal capacityId = DataChange.dataToBigDecimal(mapValue.get("capacityId"));

        //判断传入的是门岗 还是 月台 只会传入一个
        if(map.get("gatepostName") != null || map.get("gatepostId") != null){
            //如果传入的是门岗名称
            if(map.get("gatepostName") != null){
                //通过传入的门岗名称查询门岗ID
                BigDecimal gatepostId = qmsQueueResultMapper.getGatepostIdByGatepostName((String) map.get("gatepostName"));
                mapValue.put("queueLocationId", gatepostId);
            }else {
                //否则应该传入门岗ID
                mapValue.put("queueLocationId", map.get("gatepostId"));
            }
            mapValue.put("gridType", 3);
        }else {
            if(map.get("platformName") != null){
                //通过传入的月台名称查询月台ID
                BigDecimal platformId = qmsQueueResultMapper.getPlatformIdByPlatformName((String) map.get("platformName"));
                mapValue.put("queueLocationId", platformId);
            }else {
                //否则应该传入的是月台ID
                mapValue.put("queueLocationId", map.get("platformId"));
            }
            mapValue.put("gridType", 1);
        }

        //通过门岗ID或者月台ID 和 网格类型 查询对应网格ID
        BigDecimal gridId = qmsQueueResultMapper.getGridIdByGatepostIdAndGridType(mapValue);
        //查询当前网格链表还在排队中的 最大序号 + 1
        mapValue.put("gridId", gridId);
        Integer maxListNodeOrder = qmsQueueListMapper.getMaxListNodeOrder(mapValue);
        int listNodeOrder = (maxListNodeOrder == null ? 0 : maxListNodeOrder) + 1;
        //新增一条链表
        QmsQueueList qmsQueueList = new QmsQueueList();
        //添加主键ID 新增的链表Id
        BigDecimal maxId = qmsQueueListMapper.selectMaxId();
        qmsQueueList.setListId(maxId);
        //添加排队序号 逻辑删除 正在排队0 添加网格Id 添加运力Id
        qmsQueueList.setListNodeOrder(new BigDecimal(listNodeOrder));
        qmsQueueList.setDeleted(new BigDecimal(0));
        qmsQueueList.setGridId(gridId);
        qmsQueueList.setCapacityId(capacityId);

        qmsQueueList.setInsertUsername("admin");
        qmsQueueList.setInsertTime(new Date());
        int i = qmsQueueListMapper.insertSelective(qmsQueueList);
        //更新排队实绩表
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setId(resultId);
        qmsQueueResult.setListId(maxId); //添加链表ID
        qmsQueueResult.setResultTransferTime(new Date());   //添加转移时间
        qmsQueueResult.setResultTransferReason((String) map.get("resultTransferReason")); //添加转移原因
        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        return i + i1;
    }

    /**
     * 取消排队
     *  传入参数：
     *  运输订单号 或 排队实绩ID
     *  取消原因
     *  如果是司机：通过运输订单号
     *  如果是调度人员：可通过实绩Id
     * @param map
     * @return
     */
    @Override
    public int queueCancel(Map<String, Object> map) {
        int i = 0;
        if(map.get("resultIdList") != null){
            List<Object> resultIdList = (List) map.get("resultIdList");
            for (Object resultId : resultIdList) {
                Map<String, Object> resultMap = new HashMap<>();
                resultMap.put("resultId", resultId);
                resultMap.put("resultCancelReason", map.get("resultCancelReason"));
                i += queueCancelSon(resultMap);
            }
        }else if(map.get("orderNumber") != null){
            i = queueCancelSon(map);
        }
        return i;
    }

    /**
     * 取消子表
     * @param map
     * @return
     */
    public int queueCancelSon(Map<String, Object> map){
        BigDecimal resultId;
        map.putAll(qmsQueueListMapper.getQueueResultByOrderNumber(map));
        //获取实绩Id
        resultId = DataChange.dataToBigDecimal(map.get("resultId"));
        //获取链表Id
        BigDecimal listId = DataChange.dataToBigDecimal(map.get("listId"));
        // 更新链表 移出队列
        int i = updateQueueListToDie(listId);
        //更新排队实绩
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setResultId(resultId);
        qmsQueueResult.setResultCancelTime(new Date());//设置取消时间
        qmsQueueResult.setResultCancelReason((String) map.get("resultCancelReason"));//设置取消原因
        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        return i + i1;
    }

    /**
     * 排队插队 通过实绩Id进行关联
     * 实绩ID
     * 插队原因
     * @param map
     * @return
     */
    @Override
    public int queueCutInLine(Map<String, Object> map) {
        //通过实绩Id查出关联的链表Id
        map.putAll(qmsQueueListMapper.getQueueResultByOrderNumber(map));
        //获取链表Id
        BigDecimal listId = DataChange.dataToBigDecimal(map.get("listId"));
        QmsQueueList qmsQueueList = new QmsQueueList();
        //设置链表排队顺序号 为 最小的那个 -1
        //获取排队顺序号最小值
        BigDecimal gridId = qmsQueueListMapper.getGridIdByListId(listId);
        //通过网格Id匹配当前列表中最小的顺序号
        map.putAll(qmsQueueListMapper.getQueueNumAndMin(gridId));
        //计算插队后顺序号
        int listNodeOrder = DataChange.dataToBigDecimal(map.get("listNodeOrder")).intValue() - 1;
        qmsQueueList.setListId(listId);
        qmsQueueList.setListNodeOrder(new BigDecimal(listNodeOrder));
        qmsQueueList.setUpdateTime(new Date());
        qmsQueueList.setUpdateUsername("admin");
        int i = qmsQueueListMapper.updateByPrimaryKeySelective(qmsQueueList);
        //更新排队实绩
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setResultId(DataChange.dataToBigDecimal(map.get("resultId")));
        qmsQueueResult.setResultInsertTime(new Date());
        qmsQueueResult.setResultInsertReason((String)map.get("resultInsertReason"));
        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        return i + i1;
    }

    /**
     * 进厂排队结束 PDA 形式
     * 运输订单号
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int queueEndByPDA(Map<String, Object> map) {
        //通过运输订单号查询排队实绩ID 链表Id
        map.putAll(qmsQueueListMapper.getQueueResultByOrderNumber(map));
        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("resultId")); //实绩Id
        BigDecimal listId = DataChange.dataToBigDecimal(map.get("listId"));  //链表Id
        //通过链表id修改链表对应数据 将此数据移出队列 deleted = 1
        int i = updateQueueListToDie(listId);
        //给排队实绩中添加数据
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setResultId(resultId);
        qmsQueueResult.setResultEndTime(new Date());
        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        // 排队结束添加进厂作业
        tmsTruckFeign.updateEnactoryResult(map);

        return i + i1;
    }

    /**
     * 发送运输订单信息给门岗系统
     * 包含运输订单号，车牌号，门岗名称；
     * @return
     */
    public int sendMessageToGatepost(String orderNumber){
        //通过运输订单号查询相关信息
        Map<String, Object> map = qmsQueueListMapper.getOrderMesByOrderNum(orderNumber);
        map.put("orderNumber", orderNumber);
        //访问门岗接口发送数据
        //************************************
        return 0;
    }

    /**
     * 链表监控
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> listMonitor(Map<String, Object> map) {
        return qmsQueueListMapper.listMonitor(map);
    }


    /**
     * 修改状态为 1 移出链表队列
     * @param listId
     * @return
     */
    public int updateQueueListToDie(BigDecimal listId){
        QmsQueueList qmsQueueList = new QmsQueueList();
        qmsQueueList.setId(listId);
        qmsQueueList.setDeleted(new BigDecimal(1));
        return qmsQueueListMapper.updateByPrimaryKeySelective(qmsQueueList);
    }

    /**
     * 结束仓库排队
     * @param map {orderNumber 运输订单号}
     * @return
     */
    public int queueEndWarehouse(Map<String, Object> map){
        //通过运输订单号查询排队实绩ID 链表Id
        map.putAll(qmsQueueListMapper.getQueueResultByOrderNumber(map));
        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("resultId")); //实绩Id
        BigDecimal listId = DataChange.dataToBigDecimal(map.get("listId"));  //链表Id
        //通过链表id修改链表对应数据 将此数据移出队列 deleted = 1
        int i = updateQueueListToDie(listId);
        //给排队实绩中添加数据
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        qmsQueueResult.setResultId(resultId);
        qmsQueueResult.setResultEndTime(new Date());
        int i1 = qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
        return i + i1;
    }

    /**
     * 仓库排队监控
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> warehouseListMonitor(Map<String, Object> map) {
        return qmsQueueListMapper.warehouseListMonitor(map);
    }

    /**
     * 查询排队详情 门岗ID 或者 月台ID
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getListQueueMes(Map<String, Object> map) {
        HashMap<String, Object> hashMap = new HashMap<>();
        if(map.get("gatepostId") != null){//传入的是门岗ID 自然类型就为 3：进厂门岗
            hashMap.put("queueLocationId", map.get("gatepostId"));
            hashMap.put("gridType", 3);
        }else if(map.get("platformId") != null){ // 1：成品仓库
            hashMap.put("queueLocationId", map.get("platformId"));
            hashMap.put("gridType", 1);
        }
        //通过门岗ID和 类型 来查询网格ID
        BigDecimal gridId = qmsQueueResultMapper.getGridIdByGatepostIdAndGridType(hashMap);
        map.put("gridId", gridId);
        List<Map<String, Object>> listQueueMes = qmsQueueListMapper.getListQueueMes(map);
        for (Map<String, Object> queueMes : listQueueMes) {
            //获取排队开始时间
            Date resultStartTime = (Date) queueMes.get("resultStartTime");
            long sTime = new Date().getTime() - resultStartTime.getTime(); //当前时间与开始时间相差毫秒数
            sTime = sTime / (1000 * 60);
            queueMes.put("minute", sTime + "分钟");
        }
        return listQueueMes;
    }
}
